package com.factory.wx.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.factory.base.entity.aggregates.PageBean;
import com.factory.base.entity.aggregates.ResponseBean;
import com.factory.base.entity.po.benteler.BentelerPlan;
import com.factory.base.entity.po.remote.UserInfo;
import com.factory.base.util.res.ResponseBeanBuilder;
import com.factory.user.mapper.UserInfoMapper;
import com.factory.web.entity.pm.*;
import com.factory.web.entity.pm.res.ApprovalUserRes;
import com.factory.web.mapper.BentelerPlanMapper;
import com.factory.web.mapper.pm.ApprovalRuleMapper;
import com.factory.web.mapper.pm.OnsiteApprovalTaskMapper;
import com.factory.web.mapper.pm.OnsiteCalendarMapper;
import com.factory.web.mapper.pm.OnsiteTaskMapper;
import com.factory.web.service.pm.PreventMistakeService;
import com.factory.wx.entity.req.PlanApprovalTaskReq;
import com.factory.wx.entity.req.PlanInfoReq;
import com.factory.wx.entity.req.PlanPageReq;
import com.factory.wx.entity.res.PlanApprovalTaskRes;
import com.factory.wx.entity.res.PlanPassDetailRes;
import com.factory.wx.mapper.PlanMapper;
import com.factory.wx.service.PlanPassService;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author sMArT
 * @version 1.0
 * @date 2021-07-13 12:42 下午
 */
@Service("planPassService")
public class PlanPassServiceImpl implements PlanPassService {

    private final static Logger LOGGER = LoggerFactory.getLogger(PlanPassServiceImpl.class);
    /**
     * 审批规则类型为：1新建计划审批
     */
    private static int APPROVAL_TYPE_NEW = 1;
    /**
     * 审批规则类型为：3防错
     */
    private static int APPROVAL_TYPE_PM = 3;
    @Autowired
    private PlanMapper planMapper;
    @Autowired
    private BentelerPlanMapper bentelerPlanMapper;
    @Autowired
    private OnsiteCalendarMapper onsiteCalendarMapper;
    @Autowired
    private ApprovalRuleMapper approvalRuleMapper;
    @Autowired
    private UserInfoMapper userInfoMapper;
    @Autowired
    private PreventMistakeService preventMistakeService;

    @Autowired
    private OnsiteApprovalTaskMapper onsiteApprovalTaskMapper;
    @Autowired
    private OnsiteTaskMapper onsiteTaskMapper;

    private static IPage<BentelerPlan> getBentelerPlanIPage(PlanPageReq planPageReq, List<OnsiteCalendar> onsiteCalendarList, BentelerPlanMapper bentelerPlanMapper) {
        IPage<BentelerPlan> bentelerPlanIPage;
        List<Long> planIds = onsiteCalendarList.stream().map(OnsiteCalendar::getBentelerPlanId).collect(Collectors.toList());

        LambdaQueryWrapper<BentelerPlan> wrapper = new LambdaQueryWrapper<>();
        wrapper.in(BentelerPlan::getId, planIds);
        wrapper.eq(BentelerPlan::getPlanType, planPageReq.getCheckType());

        if (StringUtils.isNotEmpty(planPageReq.getName())) {
            wrapper.like(BentelerPlan::getName, planPageReq.getName());
        }
        IPage<BentelerPlan> page = new Page<>(planPageReq.getPage(), planPageReq.getSize());
        bentelerPlanIPage = bentelerPlanMapper.selectPage(page, wrapper);
        return bentelerPlanIPage;
    }


    /**
     * 计划放行列表
     *
     * @param planPageReq
     * @return
     */
    @Override
    public PageBean<BentelerPlan> getPassListByPage(PlanPageReq planPageReq) {

        LambdaQueryWrapper<OnsiteApprovalTask> approvalTaskWrapper = new LambdaQueryWrapper<>();
        approvalTaskWrapper.eq(OnsiteApprovalTask::getApprovalUserId, planPageReq.getUserId());
        approvalTaskWrapper.eq(OnsiteApprovalTask::getStatus, 0);//获取未处理过的计划

        List<OnsiteApprovalTask> onsiteApprovalTaskList = onsiteApprovalTaskMapper.selectList(approvalTaskWrapper);
        IPage<BentelerPlan> bentelerPlanIPage = new Page<>();
        if (CollUtil.isNotEmpty(onsiteApprovalTaskList)) {
            List<Long> taskIds = onsiteApprovalTaskList.stream().map(OnsiteApprovalTask::getOnsiteTaskId).collect(Collectors.toList());
            List<OnsiteTask> onsiteTaskList = onsiteTaskMapper.selectBatchIds(taskIds);
            if (CollUtil.isNotEmpty(onsiteTaskList)) {
                List<Long> calendarIds = onsiteTaskList.stream().map(OnsiteTask::getOnsiteCalendarId).collect(Collectors.toList());
                List<OnsiteCalendar> onsiteCalendarList = onsiteCalendarMapper.selectBatchIds(calendarIds);
                if (CollUtil.isNotEmpty(onsiteCalendarList)) {
                    bentelerPlanIPage = getBentelerPlanIPage(planPageReq, onsiteCalendarList, bentelerPlanMapper);
                }
            }
        }
        return PageBean.<BentelerPlan>builder().row(bentelerPlanIPage.getRecords()).total(bentelerPlanIPage.getTotal()).page(bentelerPlanIPage.getPages()).build();
    }

    /**
     * 计划放行详情
     *
     * @param planInfoReq
     * @return
     */
    @Override
    public PlanPassDetailRes getPassDetail(PlanInfoReq planInfoReq) {
        BentelerPlan bentelerPlan = bentelerPlanMapper.selectById(planInfoReq.getPlanId());
        List<OnsiteCalendar> onsiteCalendarList = onsiteCalendarMapper.selectList(Wrappers.<OnsiteCalendar>lambdaQuery()
                .eq(OnsiteCalendar::getBentelerPlanId, planInfoReq.getPlanId()));

        List<String> usersName = new ArrayList<>();
        onsiteCalendarList.forEach(onsiteCalendar -> {
            UserInfo userInfo = userInfoMapper.selectById(onsiteCalendar.getUserId());
            usersName.add(userInfo.getName());
        });

        //获取title
        OnsiteTitle title = preventMistakeService.getTitle(planInfoReq.getPlanId());
        List<PlanApprovalTaskRes> planApprovalTaskResList = planMapper.getApprovalTaskList(planInfoReq.getUserId());

        PlanPassDetailRes planPassDetailRes = PlanPassDetailRes.builder()
                .planId(bentelerPlan.getId())
                .planName(bentelerPlan.getName())
                .effectiveDate(bentelerPlan.getEffectiveDate())
                .usersName(usersName)
                .titleInfo(title)
                .onsiteCalendars(onsiteCalendarList)
                .planApprovalTasks(planApprovalTaskResList)
                .build();
        return planPassDetailRes;
    }

    /**
     * 计划放行
     *
     * @param planApprovalTaskReq
     * @return
     */
    @Override
    public ResponseBean approvalTaskOperate(PlanApprovalTaskReq planApprovalTaskReq) {

        OnsiteApprovalTask ot = new OnsiteApprovalTask();
        ot.setOnsiteTaskId(planApprovalTaskReq.getTaskId());
        ot.setReason(planApprovalTaskReq.getReason());
        ot.setStatus(planApprovalTaskReq.getStatus());
        int updateStatus = onsiteApprovalTaskMapper.update(ot, Wrappers.<OnsiteApprovalTask>lambdaQuery()
                .eq(OnsiteApprovalTask::getOnsiteTaskId, planApprovalTaskReq.getTaskId())
                .eq(OnsiteApprovalTask::getApprovalUserId, planApprovalTaskReq.getUserId()));
        if (updateStatus > 0) {
            //审批不通过触发下一级别
            if (planApprovalTaskReq.getStatus() == 2) {
                ApprovalRule approvalRuleMaxLevel = approvalRuleMapper.selectOne(Wrappers.<ApprovalRule>lambdaQuery()
                        .eq(ApprovalRule::getType, APPROVAL_TYPE_PM)
                        .orderByDesc(ApprovalRule::getApprovalLevel)
                        .last("limit 1"));
                //获取当前审批级别
                OnsiteApprovalTask onsiteApprovalTask = onsiteApprovalTaskMapper.selectOne(Wrappers.<OnsiteApprovalTask>lambdaQuery()
                        .eq(OnsiteApprovalTask::getOnsiteTaskId, planApprovalTaskReq.getTaskId())
                        .eq(OnsiteApprovalTask::getApprovalUserId, planApprovalTaskReq.getUserId()).last("limit 1"));
                int userLevel = onsiteApprovalTask.getApprovalLevel();
                if (userLevel < approvalRuleMaxLevel.getApprovalLevel()) {
                    ApprovalUserRes approvalUserRes = preventMistakeService.getApprovalUserInfo(userLevel + 1, APPROVAL_TYPE_PM);
                    onsiteApprovalTaskMapper.insert(OnsiteApprovalTask.builder()
                            .onsiteTaskId(onsiteApprovalTask.getOnsiteTaskId())
                            .approvalLevel(approvalUserRes.getApprovalLevel())
                            .approvalUserId(approvalUserRes.getUserId())
                            .status(0)
                            .build());
                }
            }
            //审批通过
            if (planApprovalTaskReq.getStatus() == 1) {
                //审批通过，保存审批历史
                preventMistakeService.getApprovalUserInfo(1, APPROVAL_TYPE_PM);
            }
        }

        return ResponseBeanBuilder.ok();
    }

}
